@@ -15,7 +15,7 @@ option add *Text.c_link			blue	startup
 proc install_init { appName dotFile } {
     global install
     set install(appName) $appName
-    install_progVar wish /usr/local/bin/wish {wish absolute pathname}
+    install_progVar wish [installGuessPath /usr/local/bin/wish wish] {wish absolute pathname}
     set install(dotFile) $dotFile
     if [file readable $dotFile] {
 	if [catch {uplevel #0 source $dotFile} msg] {
@@ -159,6 +159,36 @@ proc installFieldDefault { item {override 0} } {
     }
 }
 
+proc installGuessDir { defpath file } {
+    global env
+    set tmp $defpath
+
+    puts "installGuessDir: def=$defpath, file=$file..."
+    foreach dir [split $env(PATH) :] {
+	if {[file exists $dir/$file]} {
+	    set tmp $dir
+	    break
+	}
+    }
+    puts "returning $tmp\n"
+    return $tmp
+}
+
+proc installGuessPath { defpath file } {
+    global env
+    set tmp $defpath
+
+    #puts "installGuessPath: def=$defpath, file=$file..."
+    foreach dir [concat /etc /usr/lib/nmh [split $env(PATH) :]] {
+	if {[file exists $dir/$file]} {
+	    set tmp $dir/$file
+	    break
+	}
+    }
+    #puts "returning $tmp\n"
+    return $tmp
+}
+
 proc install_help { text } {
     global install
     set install(helpText) $text
@@ -219,18 +249,12 @@ proc installFieldDone {} {
     }
 }
 
-proc installSetValue { _var } {
+proc installSetValue { varName } {
     global install
-    if [info exists install(entry,$_var)] {
-	set _value [$install(entry,$_var) get]
-	if [string match *(* $_var] {
-	    set _arrayName [lindex [split $_var (] 0]
-	    global $_arrayName
-	} else {
-	    global $_var
-	}
-	set $_var $_value
-#	installFeedback "$_var $_value"
+    if [info exists install(entry,$varName)] {
+	set value [$install(entry,$varName) get]
+	upvar #0 $varName var
+	set var $value
     }
 }
 proc installGetValue { var } {
@@ -266,6 +290,9 @@ proc installVerify {} {
 	    if ![file isdirectory $path] {
 		set willMakeDir 0
 		foreach dirType $install(dirlist) {
+		    if {![info exist install(dir,$dirType)]} {
+			installSetValue install(dir,$dirType)
+		    }
 		    set newdir $install(dir,$dirType)
 		    if {$newdir == $path} {
 			set willMakeDir 1
@@ -569,6 +596,8 @@ proc installInner { {logProc nolog} } {
 		    if [file isdirectory $f] {
 			installCmd $logProc [list rm -rf $dir/$newf]
 			installCmd $logProc [list cp -r $f $dir/$newf]
+                        installCmd $logProc [list chmod a+rx $dir/$newf]
+                        installCmd $logProc [list chmod -R a+r $dir/$newf]
 		    } else {
 			installCmd $logProc [list rm -f $dir/$newf]
 			installCmd $logProc [list cp $f $dir/$newf]
@@ -614,6 +643,7 @@ proc installFake {} {
     global exmh install
     toplevel .fake
     wm title .fake "Install Actions"
+    wm iconname .fake "Install Actions"
     frame .fake.top
     button .fake.top.quit -text "Dismiss" -command {destroy .fake}
     label .fake.top.label -text "  Pending install actions"
@@ -632,9 +662,10 @@ proc installFake {} {
     installInner log
 }
 proc install_dialog {} {
-    global install tk_version
+    global install
 
     wm title . "Exmh Install"
+    wm iconname . "Exmh Install"
     wm minsize . 200 200
 
     update idletasks
@@ -741,9 +772,10 @@ proc installBindInit {} {
 
 }
 proc installBindKeys {} {
-    global install tk_version
+    global install
     toplevel .keys
     wm title .keys "Install Edit Preferences"
+    wm iconname .keys "Install Edit Preferences"
     frame .keys.b
     pack .keys.b -side top -fill x
     button .keys.b.quit -text Dismiss -command {destroy .keys}
@@ -811,7 +843,7 @@ proc installBindEntry { {list {}} } {
     }
 }
 proc installBindEntryInner { what } {
-    global install tk_version
+    global install
 
     # Modification bindings
 
@@ -827,11 +859,7 @@ proc installBindEntryInner { what } {
 	catch {%W delete sel.first sel.last}
     }
 
-    if {$tk_version >= 4.0} {
-	set bsProc tkEntryBackspace
-    } else {
-	set bsProc tk_entryBackspace
-    }
+    set bsProc tkEntryBackspace
     foreach bs {backspace backspace2 backspace3} {
 	bind $what $install(key,$bs) "$bsProc %W"
     }
