@@ -150,8 +150,49 @@ proc Edit_Dialog {draftID} {
 proc EditDialog {draftID} {
     Edit_Dialog $draftID
 }
+proc EditAddPassPhrasePane {id w} {
+    global pgp
+    if {$pgp(enabled) && $pgp(seditpgp)} {
+
+	# seditpgp main area
+	Pgp_SetSeditPgpName $pgp($pgp(version,$id),myname,$id) $id
+	set pgp(fullName,$id) $pgp($pgp(version,$id),fullName)
+
+	if {![winfo exists $w.pgp]} {
+	    pack [frame $w.pgp] -side bottom -fill x -ipady 2
+	}
+	if {![winfo exists $w.pgp.l1]} {
+	    pack [label $w.pgp.l1 -text "Passphrase for "] \
+		    -side left
+	}
+	if {![winfo exists $w.pgp.b]} {
+	    pack [button $w.pgp.b -textvariable pgp(sedit_label,$id) \
+		    -command "Pgp_SetMyName \$pgp(version,$id) $id"] \
+		    -side left -ipady 2
+	}
+	if {![winfo exists $w.pgp.l2]} {
+	    pack [label $w.pgp.l2 -text ": "] \
+		    -side left
+	}
+	if {![winfo exists $w.pgp.e]} {
+	    set v $pgp(version,$id)
+	    pack [entry $w.pgp.e -textvariable pgp(cur,pass,$id) -show *] \
+		    -side left -expand yes -fill x -ipady 2
+	}
+
+	# Add extras if requested
+	if {$pgp(seditpgpextras)} {
+	    if {![winfo exists $w.pgp2]} {
+		pack [frame $w.pgp2] -side bottom -fill x
+	    }
+	    if {![winfo exists $w.pgp2.l1]} {
+		pack [label $w.pgp2.l1 -textvariable pgp(sedit_label2,$id)] -side left
+	    }
+	}
+    }
+}
 proc EditShowDialog {id text} {
-    global exwin editor
+    global exwin editor pgp
     # Create the buttons and arrange them from left to right in the RH
     # frame. Embed the left button in an additional sunken frame to indicate
     # that it is the default button.
@@ -161,8 +202,16 @@ proc EditShowDialog {id text} {
 	wm transient $d
 	$d config -relief raised -borderwidth 2
 
+	# PGP version-setting moved out from seditpgp code
+	if {$pgp(enabled)} {
+	    if {![info exists pgp($pgp(version,$id),myname,$id)]} {
+		set pgp($pgp(version,$id),myname,$id) $pgp($pgp(version,$id),myname)
+	    }
+	    EditAddPassPhrasePane $id $d
+	}
+
 	foreach but [Widget_GetButDef $d] {
-	    Widget_AddButDef $d $but
+	    Widget_AddButDef $d $but {right}
 	    Widget_ReEvalCmd $d.$but	;# expand $id variable now
 	}
 	catch {pack $d.abort -padx 15}
@@ -173,7 +222,7 @@ proc EditShowDialog {id text} {
 	    ButtonMenuInner $menu	;# This also expands variables
 	}
 
-	Widget_Message $d msg -text "$text\nReturn for Send\nControl-c for Kill" -aspect 300
+	Widget_Message $d msg -text "$text\nReturn for Send\nControl-c for Kill" -aspect 400
 
 	focus $d
 	bind $d <Return> [list $d.send invoke]
@@ -190,15 +239,10 @@ proc EditShowDialog {id text} {
     }
 }
 proc EditDialogMsg {id msg} {
-    global tk_version
     set d .edit$id
     catch {destroy $d.f}
     Widget_Frame $d f
-    if {$tk_version >= 3.3} {
-	pack $d.f -before [lindex [pack slaves $d] 0] -side bottom
-    } else {
-	pack before [lindex [pack info $d] 0] $d.f {bottom}
-    }
+    pack $d.f -before [lindex [pack slaves $d] 0] -side bottom
     set lines [llength [split $msg \n]]
     Widget_Text $d.f [expr {$lines > 8 ? 8 : $lines}]
     $d.f.t insert 1.0 $msg
@@ -213,10 +257,18 @@ proc EditDialogDone {act id {hide hide}} {
 
 proc EditStart { draft {type prog} } {
     # Start the editor, reusing an existing session if possible
-    global editor exmh mhProfile
+    global editor exmh mhProfile pgp
 
     Exmh_Debug EditStart $draft $type
 
+    if $pgp(enabled) {
+	# Copy the default PGP values into this window
+	set id [SeditId $draft]
+	foreach var {encrypt sign mime clearsign format version} {
+	    set pgp($var,$id) $pgp($var)
+	}
+    }
+    
     switch -glob -- $editor($type) {
 	*mxedit* {
 	    if ![info exists exmh(editInterp)] {
