@@ -89,35 +89,65 @@ folders without committing moves and delete operations.
 	{faces(enabled) facesEnabled ON {Use faces database}
 "Search for and display images from the facesaver database.
 (Faces Preferences.)"}
-
-        {   addr_db(enabled)
+	{
+            addr_db(enabled)
             addressdbEnabled
             ON
-            "Enable address saving"
-"If set, From addresses are remembered and available in an address
-browser.  Type <Control-Tab> in \"sedit\" to get address completion.
-(Address Database Preferences.)" }
+            "Automatic address saving"
+            "If set, From addresses are remembered and available in an address browser."
+	}
 
     }
 }
 
+proc SlowDisplay_Init {} {
+    global exmh
+
+    Preferences_Add "Slow Display" \
+"These items determine which parts of the system to disable when you have a 
+slow display" {
+        {exmh(slowDispLimit) slowDispLimit 200000	{Slow Display Limit}
+"This is the microsecond count used to determine if the user is on a
+\"slow display\"."}
+        {exmh(slowDispFaces) slowDispFaces OFF {Show faces on a slow display}
+	"Do faces processing if the display is slow"}
+        {exmh(slowDispIcons) slowDispIcons OFF {Show color icons on a slow display}
+	"Use color icons if the display is slow."}
+    }
+    Preferences_Resource exmh(testglyph) testGlyph flagdown.gif
+    if ![string match /* exmh(testglyph)] {
+	set exmh(testglyph) $exmh(library)/$exmh(testglyph)
+    }
+    set time [lindex [time {
+	toplevel .foo
+	pack [canvas .foo.c]
+	image create photo testicon -file $exmh(testglyph)
+	.foo.c configure -width [image width testicon] \
+		-height [image height testicon]
+	destroy .foo
+    }] 0]
+    set exmh(slowDisp) [expr {$time > $exmh(slowDispLimit)}]
+    if $exmh(slowDisp) {
+	Exmh_Debug "Slow display: $time"
+    } else {
+	Exmh_Debug "Fast display: $time"
+    }
+}
+
 proc Faces_Init {} {
-    global faces tk_version
+    global faces
     if {$faces(dir) == {}} {
 	set faces(enabled) 0
     }
     # faces(suffix) starts out (by default) with {xpm gif xbm}
-    if {![info exists faces(suffix)] || ([llength $faces(suffix)] == 0) ||
-	($tk_version < 4.0)} {
+    if {![info exists faces(suffix)] || ([llength $faces(suffix)] == 0)} {
 	set faces(suffix) xbm
     }
-    if {$tk_version >= 4.0} {
-	# Double check the non-standard pixmap image type to avoid file stat
-	if {[lsearch [image types] pixmap] < 0} {
-	    set ix [lsearch $faces(suffix) xpm]
-	    if {$ix >= 0} {
-		set faces(suffix) [lreplace $faces(suffix) $ix $ix]
-	    }
+    # Double check the non-standard pixmap image type to avoid file stat
+    if {[lsearch [image types] pixmap] < 0} {
+	set ix [lsearch $faces(suffix) xpm]
+	if {$ix >= 0} {
+	    set faces(suffix) [lreplace $faces(suffix) $ix $ix]
 	}
     }
 
@@ -126,6 +156,9 @@ proc Faces_Init {} {
 
 Any given mail address can match a range of face images, from most specific (such as an X-Face) or most general (such as an a default image for a toplevel domain)." {
 
+        {faces(xFaceEnabled) xFaceEnabled ON {Use X-Face headers}
+	"If a message contains an X-Face header, display the image encoded into the header."}
+
 	{faces(xfaceProg) xfaceProg {} {X-Face command}
 "A command to convert an X-Face: header line into a displayable X11 bitmap.
 
@@ -140,6 +173,9 @@ Defining this enables the display of X-Face images.  This is independent of the
 	{faces(rowEnabled) facesRow OFF {Use faces row}
 "Use a horizontal row to display all the matching images of a face, rather than the most specific one."}
 
+        {faces(xImageUrl) xImageUrl ON {Use X-Image-URL}
+"If a message contains an X-Image-URL header, display the image referenced in that header."}
+
 	{faces(defer) facesDefer ON {Background processing} 
 "When on, this causes faces display to be handled as a background task.  This allows the display of a faces row to be aborted when another message is selected.  If your machine can display faces quickly enough, you should turn this off."}
 
@@ -168,7 +204,7 @@ proc Faces_Create { parent } {
 }
 
 proc Faces_Setup args {
-    global faces faceCache
+    global faces faceCache exmh
 
     # should call this when one of these changes:
     #	faces(path), faces(dir), faces(sets), env(FACEPATH)
@@ -194,7 +230,7 @@ proc Faces_Setup args {
 	$faces(parent) config -width 0
     }
 
-    if $faces(rowEnabled) {
+    if {$faces(rowEnabled) && (!$exmh(slowDisp) || $exmh(slowDispFaces))} {
 	set row [Widget_Frame $faces(rowparent) faceRow Face {top fill}]
 	pack $row -before $faces(parent) -side bottom
 
@@ -209,7 +245,7 @@ proc Faces_Setup args {
 			-bitmap @$exmh(library)/exmh.bitmap]
 
     # kludge to get default background of the labels
-    if $faces(rowEnabled) {
+    if {$faces(rowEnabled) && (!$exmh(slowDisp) || $exmh(slowDispFaces))} {
 	set f [FaceAlloc]
 	set faces(facebg) [lindex [$f config -bg] 4]
 	Face_Delete
@@ -282,7 +318,7 @@ when replying." }
 "The filename of the quote file relative to the exmh home directory,
 which is usually the user's home directory.  The default is @, I
 have mine set to Mail/@." }
-    {quote(symlink) quoteSymlink ON {Only symlink}
+    {quote(symlink) quoteSymlink OFF {Only symlink}
 "In previous versions of exmh, the quote file was only a symlink to
 the message being replied to.  Select this if you want to override
 the following options and keep this behavior." }
@@ -417,6 +453,9 @@ is interactive, then run it from exmh-async:
 exmh-async xterm -e ispell
 (This is faked - exmh-async isn't really used.  Instead a temporary
 wish script is used.)"}
+    {sedit(autosaveInterval)   seditAutosaveInterval 60 {Seconds between auto saves}
+"This is the interval, in seconds, between automatic saves of 
+the message being edited.  Set to 0 to disable autosaving."}
     }
     Preferences_Resource sedit(wordbreakpat) seditWordBreakPat "\[\ \t/>\]"
     # Converting from boolean to choice
@@ -452,161 +491,84 @@ proc Sedit_CheckPoint {} {
 proc Pgp_Init {} {
     global pgp env miscRE
 
-    set miscRE(headerend) {^(--+.*--+)?$}
-    set miscRE(mimeheaders) {^content-[-a-z]+:}
-    set miscRE(true) {^(on|y(es)?|t(rue)?)$}
-    set miscRE(beginpgp) {^-+BEGIN PGP}
-    set miscRE(beginpgpkeys) {^-+BEGIN PGP PUBLIC KEY BLOCK-+$}
-    set miscRE(beginpgpclear) {^-+BEGIN PGP SIGNED MESSAGE-+$}
+    # Load a minimal amount of data
+    # Otherwise it cannot check for pgp
+    Pgp_Base_Init
 
-    # Set pgp message colors
-    if {[tk colormodel .] == "color"} {
-	Preferences_Resource pgp(BadSig) m_pgpBadSig \
-	    "-foreground red"
-	Preferences_Resource pgp(GoodUntrustedSig) \
-	    m_pgpGoodUntrustedSig "-foreground blue"
-	Preferences_Resource pgp(GoodTrustedSig) \
-	    m_pgpGoodTrustedSig "-foreground darkgreen"
-	Preferences_Resource pgp(OtherMsg) \
-	    m_pgpOtherMsg {}
-    } else {
-	Preferences_Resource pgp(BadSig) m_pgpBadSig {}
-	Preferences_Resource pgp(GoodUntrustedSig) \
-	    m_pgpGoodUntrustedSig {}
-	Preferences_Resource pgp(GoodTrustedSig) \
-	    m_pgpGoodTrustedSig {}
-	Preferences_Resource pgp(OtherMsg) m_pgpOtherMsg {}
+    # Set up exmh for a pgp version
+    # if there is an appropriate keyring
+    foreach v $pgp(supportedversions) {
+        if { [file exists [set pgp($v,pubring)]] && 
+             [file isdirectory [set pgp($v,path)]]} {
+            set pgp($v,enabled) 1
+            set pgp(enabled) 1
+            lappend setup $v
+        }
+    }
+    if { ![info exists setup] } {
+        return
     }
 
-    # figure out the path for pgp files
-    if [info exists env(PGPPATH)] {
-	set pgp(pgppath) $env(PGPPATH)
-    } elseif [file isdirectory $env(HOME)/.pgp] {
-	set pgp(pgppath) $env(HOME)/.pgp
-    } else {
-	set pgp(pgppath) [pwd]
+    # Now that we know, that there is a pgp variant
+    # installed on the system, we load all the stuff
+    # every pgp version needs for basic functionality
+    Pgp_Shared_Init    
+
+# Global PGP preferences
+    Preferences_Add "General PGP Interface" \
+"Pretty Good Privacy (PGP) lets you sign and encrypt messages using 
+public keys.
+There is considerable documentation that comes with PGP itself.
+This set of preferences controls the general behavior of all the
+PGP modules." {
+    {pgp(seditpgp) pgpSeditPgp OFF {Sedit PGP passphrase}
+"Turning this on provides you with a PGP passphrase field in the sedit
+window so that you will not be prompted with the passphrase prompt.  
+Changing this value will require that you exit and re-enter exmh if 
+you've already composed email." }
+    {pgp(seditpgpextras) pgpSeditPgpExtras OFF {Sedit PGP extras info}
+"Turning this on gives extra information about what key and PGP
+version are currently selected.  Users with multiple PGP keys or PGP
+versions may find it helpful.  Changing this value will require that 
+you exit and re-enter exmh if you've already composed email." }
+    {pgp(sign) pgpSign ON {Sign outgoing messages}
+"If this is turned on, outgoing messages will be signed." }
+    {pgp(encrypt) pgpEncrypt OFF {Encrypt outgoing messages}
+"If this is turned on, outgoing messages will be encrypted." }
+    {pgp(mime) pgpMime ON {Use Mime for PGP}
+"If this is turned on, outgoing PGP messages will use Mime for encapsulation." }
+    {pgp(clearsign) pgpClearSign ON {Sign messages in the clear}
+"If this is turned on, messages will be signed so that they can be read by
+non-PGP mail readers." }
+    {pgp(format) pgpFormat {CHOICE pm plain app} {Format to encode PGP}
+"There are multiple standards for PGP encoding.
+    Pm:     Use the multipart/pgp standard (This is the preferred standard)
+    Plain:  No MIME headers at all
+    App:    Use the now deprecated application/pgp standard.
+This can be changed on the fly from the sedit window" }
+    {pgp(version) pgpVersion {CHOICE pgp pgp5 gpg} {Version of PGP for new messages}
+"There are multiple versions of the PGP program.
+    PGP:    Pretty Good Privacy, Version 2
+    PGP5:   Pretty Good Privacy, Version 5
+    GPG:    GNU Privacy Guard
+This can be changed on the fly from the sedit window" }
+    {pgp(passtimeout) pgpPassTimeout 60 {Minutes to cache PGP passphrase}
+"Exmh will clear its memory of PGP passphrases after
+this time period, in minutes, has elapesed." }
     }
 
-    # if the user doesn't have public key ring, pgp isn't setup: give up !
-    set pgp(pubring) $pgp(pgppath)/pubring.pgp
-    if [file exists $pgp(pubring)] {
-	set pgp(enabled) 1
-    } else {
-	set pgp(pubring) {}
-	set pgp(enabled) 0
-	return
+    # And now load the version specific stuff
+    foreach v $setup {
+        # Load version specific support
+        Pgp_${v}_Init
+        # Add version specific Preferences
+        Pgp_Preferences $v
     }
 
-    set pgp(menutext,signclear) "Check the signature"
-    set pgp(menutext,signbinary) "Show content"
-    set pgp(menutext,encrypt) "Show content"
-    set pgp(menutext,encryptsign) "Show content"
-    set pgp(menutext,keys-only) "Show content"
-    
-    set pgp(decode,none) 0
-    set pgp(decode,all) 1
-    set pgp(decode,keys) {$action == "keys-only"}
-    set pgp(decode,signed) {![regexp {encrypt} $action]}
+    # Other initialization
+    Pgp_Match_Init
+    Pgp_Exec_Init
 
-#    {pgp(fullkeyring) pgpFullKeyRing OFF {Potentially search the full keyring}
-#"When matching, exmh does a first strict selection based on the hostname 
-#and then looks for the best match in this selection. If the first selection
-#doesn't select anything, this option allows exmh to search the key in
-#the whole keyring instead of giving up and query the user." }
-
-    Preferences_Add "PGP interface" \
-"PGP is the Pretty Good Privacy package from Zimmerman.
-PGP lets you sign and encrypt messages using public keys.
-There is considerable documentation that comes with PGP itself." {
-    {pgp(keeppass)  pgpKeepPass  ON {Keep PGP password}
-"Exmh tries to remember your PGP password between pgp
-invocations. But the password is then kept in a global
-variable, which is not safe, because of \"send\"'s power.
-If you turn this feature off, exmh will use xterm to run
-pgp so that it doesn't have to deal with the password at all." }
-    {pgp(echopass)  pgpEchoPass  ON {Echo '*' when typing pass}
-"If you have pgpKeepPass on, Exmh will prompt for your pgp password.
-A * will be echoed for every character typed depending on this option."}
-    {pgp(grabfocus)  pgpGrabFocus  ON {Password dialog grabs input focus}
-"When exmh prompts for the PGP password it will globally grab input 
-focus if this is on.  Some users like it because they don't need to
-select the popup dialog or because it lessens the risk they will type
-their password in the wrong window.  It annoys or does not work for
-other people." }
-    {pgp(passtimeout)  pgpPassTimeout  60 {Minutes to cache PGP password}
-"Exmh will clear its memory of the PGP password after
-this time period, in minutes, has elapesed.  If you use
-different keys, they have their own timeout period." }
-    {pgp(rfc822) pgpRfc822 OFF {Encrypt headers}
-"Used to encrypt the whole message, instead of only encrypting
-the body, so that the subject line (for instance) is also
-safely transmitted." }
-    {pgp(choosekey) pgpChooseKey ON {Always choose the sign-key}
-"When signing a message, sedit can either use the default key or ask
-the user to choose which key he wants to use. Of course, if you only
-have 1 private key this setting doesn't interest you much." }
-    {pgp(useexpectk) pgpUseExpectk OFF {Use expectk if available}
-"Expectk is a utility that can commuicate interactively with both
-PGP and exmh.  With this option enabled, messages will take longer
-to decrypt, but exmh will use the correct pass phrase.  Do not turn
-this on unless you have \"Separate background process\" in
-Preferences->Background Processing on.  This is recommended if you
-have more than one secret key." }
-    {pgp(cacheids) pgpCacheIds {CHOICE persistent temporary none} {Cache map from email to public-key}
-"The way exmh figures out the public-key to use for an email address is
-often slow. This option allows you to cache the result of the matching
-so that it doesn't have to be done over and over. Furthermore the cache
-can be saved in a file .matchcache in your pgp directory so as to make
-it persistent accross exmh sessions." }
-    {pgp(minmatch) pgpMinMatch 75 {Minimum match correlation (in percents)}
-"When trying to find the key corresponding to an email address,
-exmh tries to be 'smart' and does an approximate matching. If the 
-match's quality is better than the specified percentage, exmh will
-assume it's the right key. Else it will query the user. Hence, a
-value greater than 100 will make exmh always query the user." }
-    {pgp(showinline) pgpShowInline {CHOICE none keys signed all} {Show pgp messages inline}
-"controls which pgp parts get automatically decoded with pgp. Since
-decoding generally takes time, and since clear signed messages can
-be viewed without pgp, it makes sense to limit the decoding to rare
-cases like key parts:
- - keys: only auto-decode key parts
- - signed: auto-decode key and signed parts" }
-    {pgp(shortmsgs) pgpShortMessages OFF {Short PGP reports}
-{If this is selected Exmh tries to report PGP results in one line, 
-otherwise more of the text from PGP itself is used.  For example:
-
-Good signature from user "John Smith <jsmith@well.com>".
-Signature made 1996/12/30 16:34 GMT
-
-WARNING:  Because this public key is not certified with a trusted
-signature, it is not known with high confidence that this public key
-actually belongs to: "John Smith <jsmith@well.com>".
-
-versus
-
-Good untrusted signature from "John Smith <jsmith@well.com>".}}
-    {pgp(autoextract) pgpAutoExtract ON {Extract keys automatically}
-"When you receive a keys-only part, you can have its content
-displayed and you can extract its content into your public
-key ring. The extraction can be safely done automatically,
-but you might prefer doing it manually, with a menu entry
-on the keys-only part." }
-    {pgp(keyserver) pgpKeyServer "pgp-public-keys@keys.pgp.net" {Favorite public key server}
-"When a signature check fails because of a missing key,
-exmh allows you to ask a key server for the key.
-Please select a key server that's close to
-you so as to spread the load.  Examples include
-public-key-server@pgp.mit.edu
-pgp-public-keys@keys.pgp.net
-pgp-public-keys@keys.us.pgp.net
-There are pgp-public-keys@keys.*.pgp.net servers for these domains:
-ch de es fi kr nl no uk us pt se hr tw pl
-See also http://www.pgp.net/pgpnet/
-" }
-    }
-
-    PgpExec_Init
-    PgpWWW_Init
 }
 
 # Glimpse_Init
@@ -634,11 +596,11 @@ proc Glimpse_Init {} {
 	if [info exists glimpse(init)] { return }
 
 	if [catch {exec $glimpse(path)/glimpse -V} voutput] {
-	    Exmh_Debug "$voutput"
+	    ExmhLog "$voutput"
 	    return
 	}
 	if {! [regexp {[0-9]\.[0-9]} $voutput glimpse(version)] } {
-	    Exmh_Debug "glimpse version info error : $voutput"
+	    ExmhLog "glimpse version info error : $voutput"
 	    return
 	}
 	set glimpse(init) 1
@@ -814,6 +776,16 @@ Warp:        Raise then change to the folder clicked on
 Warp & Show: Warp then select the first unseen message"
     }
 
+    { unseenwin(b2mode) unseenWinButton2
+      {CHOICE Nothing Inc}
+      {Button 2 Mode}
+"The behaviour when you press mouse button 2 depends on
+this setting.
+
+Nothing:     Does nothing
+Inc:         Just like clicking the Inc button"
+    }
+
     { unseenwin(mb1mode) unseenWinModifiedButton1
       {CHOICE None Raise Warp {Warp & Show}}
       {Modified Button 1 Mode}
@@ -839,6 +811,7 @@ Compose:     Starts a mail composition"
     }
   }
 
+  set unseenwin(digits) 1
   trace variable unseenwin(on)            w UnseenWinToggle
   trace variable unseenwin(icon)          w UnseenWinToggleIcon
   trace variable unseenwin(font)          w UnseenWinChangeFont
