@@ -81,7 +81,7 @@ proc HtmlWindow {t} {
 	    Widget_AddButDef $url $b
 	}
 	foreach M [Widget_GetMenuBDef $url] {
-	    set menu [Widget_AddMenuBDef $url $M {right padx 1}]
+	    set menu [Widget_AddMenuBDef $url $M {right padx 1 filly}]
 	    ButtonMenuInner $menu
 	}
 	global window
@@ -147,7 +147,8 @@ proc Html_Display {markup base} {
     set html(win) [HtmlWindow .html]
     upvar #0 HM$html(win) var
     set var(S_url) $base
-    Url_DisplayHtml $html(win) $base $markup
+    # Do this *after* the environment is complete.
+    after 1 [list Url_DisplayHtml $html(win) $base $markup]
     return $html(win)
 }
 proc Html_MimeShow {win part} {
@@ -197,7 +198,6 @@ proc Html_Stop {win} {
 }
 proc HtmlHit {win x y} {
     upvar #0 HM$win var
-    global uri
     lassign {href name} [UrlGetLink $win $x $y]
     UrlResolve $var(S_url) href
     URI_StartViewer $href
