@@ -34,6 +34,18 @@ your interaction with send to complete."} \
 "When \"Send in xterm window\" is selected,
 this option controls extra parameters provided
 to the xterm program to control how it is started."} \
+	{mhProfile(forwtweak) forwTweak ON {Tweak subject lines of forwarded messages}
+"If this option is enabled, the subject line of forwarded messages
+will be tweaked, in a similar manner to the prefixing of \"Re:\" to
+the subject of replies.  This is only performed if the draft message
+does not already contain a subject line (or if it is empty), as given
+in your forwcomps file."} \
+	{mhProfile(forwsubj) forwSubj {$subj (fwd)} {Subject line for forwarded messages}
+"When \"Tweak subject lines of forwarded messages\" is enabled, this
+option specifies the particular tweak to perform.  This usually consists
+of suffixing \"(fwd)\" or prefixing \"Fw:\" (both of which are removed
+if present in the original subject line).  The variable \$subj here is
+replaced with the subject of the original message."} \
 	[list mhProfile(delprefix) delPrefix [MhBackup] {Prefix of rmm'd files} \
 "The Delete operation in MH really only renames a message file to have
 a prefix like # or , (comma).  This prefernce setting is used to
@@ -143,6 +155,48 @@ proc Mh_ReplyAllSetup { folder msg } {
     }
     MhAnnoSetup $folder $msg repl
 }
+proc Mh_Forw_MungeSubj { folder msgs } {
+    global mhProfile
+    set draftID [Mh_Cur $mhProfile(draft-folder)]
+    if ![catch {eval exec scan +$folder -format "%{subject}" $msgs} subj] {
+	# just take the first line of $subj (in case of >1 messages)
+	set subj [lindex [split $subj "\n"] 0]
+	# strip off leading and trailing "fw:", "(fwd)", "<fwd>" and whitespace
+	regsub -nocase "^(\[ 	\]*((fwd?:)|(\\(fwd?\\))|(<fwd?>)))*" $subj {} subj
+	regsub -nocase "(((\\(fwd?\\))|(<fwd?>))\[ 	\]*)*$" $subj {} subj
+	set subj [string trim $subj]
+	# quote any rogue \'s or &'s in the subject line
+	regsub -all {\\} $subj {\\\\} subj
+	regsub -all {&} $subj {\\\&} subj
+	# now do the required munging, and quote \'s and &'s again
+	regsub -all {\$subj} $mhProfile(forwsubj) $subj subj
+	regsub -all {\\} $subj {\\\\} subj
+	regsub -all {&} $subj {\\\&} subj
+	catch {
+	    set fd [open $mhProfile(path)/$mhProfile(draft-folder)/$draftID r]
+	    set msgtxt [read $fd]
+	    close $fd
+	    if [regexp -indices "\n(--+)?(\n|\$)" $msgtxt posn] {
+		set cpos [lindex $posn 0]
+		set hdrtxt [string range $msgtxt 0 [expr $cpos-1]]
+		set bodytxt [string range $msgtxt $cpos end]
+	    } else {
+		set hdrtxt $msgtxt
+		set bodytxt {}
+	    }
+	    unset msgtxt
+	    if [regexp "^|\n\[Ss\]ubject:" $hdrtxt] {
+		regsub "(^|\n)(\[Ss\]ubject:)\[ 	\]*(\n|\$)" $hdrtxt "\\1\\2 $subj\\3" nhdrtxt
+	    } else {
+		set nhdrtxt "$hdrtxt\nSubject: $subj"
+	    }
+	    set fd [open $mhProfile(path)/$mhProfile(draft-folder)/$draftID w]
+	    puts -nonewline $fd $nhdrtxt
+	    puts -nonewline $fd $bodytxt
+	    close $fd
+	}
+    }
+}
 proc Mh_ForwSetup { folder msgs } {
     global mhProfile exmh
     if [file exists $mhProfile(path)/$exmh(folder)/forwcomps] {
@@ -153,6 +207,9 @@ proc Mh_ForwSetup { folder msgs } {
 	eval {MhExec forw +$folder} $msgs -nowhatnowproc
     }
     MhAnnoSetup $folder $msgs forw
+    if {$mhProfile(forwtweak)} {
+	Mh_Forw_MungeSubj $folder $msgs
+    }
 }
 proc Mh_DistSetup { folder msg } {
     global exmh mhProfile
@@ -160,7 +217,7 @@ proc Mh_DistSetup { folder msg } {
     MhExec dist +$folder $msg -nowhatnowproc
     MhAnnoSetup $folder $msg dist
 }
-proc MhAnnoSetup { folder msg key } {
+proc MhAnnoSetup { folder msg key {args {}} } {
     global mhProfile exmh
     set draftID [Mh_Cur $mhProfile(draft-folder)]
     set exmh($draftID,mhaltmsg) $mhProfile(path)/$folder/$msg
@@ -169,11 +226,19 @@ proc MhAnnoSetup { folder msg key } {
     set exmh($draftID,mhmessages) $msg
     set exmh($draftID,action) $key
     Exmh_Debug MhAnnoSetup action $key for $draftID
-    if {$exmh(anno,$key)} {
-	set exmh($draftID,mhanno$key) 1
-    }
-    if {$exmh(inplace,$key)} {
-        set exmh($draftID,mhinplace) 1
+
+	# I don't assume both alternative options will be set together
+	set noannoIX [lsearch $args -noannotate]
+	set annoIX [lsearch $args -annotate]
+	if { ($exmh(anno,$key) || ($annoIX >= 0)) &&  ($noannoIX < 0) } {
+		set exmh($draftID,mhanno$key) 1
+	}
+
+	set noinplaceIX [lsearch $args -noinplace]
+	set inplaceIX [lsearch $args -inplace]
+    if { ($exmh(inplace,$key) || ($inplaceIX >= 0)) && \
+		 ($noinplaceIX < 0) } {
+			set exmh($draftID,mhinplace$key) 1
     }
 }
 proc Mh_AnnoEnviron { draftID } {
@@ -420,9 +485,14 @@ proc Mh_SequenceUpdate { f how seq {msgs {}} } {
 	    Exmh_Debug New sequences: $mhProfile(path)/$f/$mhProfile(mh-sequences)
 	    Exmh_Debug $new
 	    puts -nonewline $out $new
-	    close $out
-	    Mh_Rename $mhProfile(path)/$f/$mhProfile(mh-sequences).new \
-		    $mhProfile(path)/$f/$mhProfile(mh-sequences)
+	    if [catch {
+		close $out
+	    } err] {
+		Exmh_Status "Cannot close $mhProfile(path)/$f/$mhProfile(mh-sequences).new: $err"
+	    } else {
+		Mh_Rename $mhProfile(path)/$f/$mhProfile(mh-sequences).new \
+			$mhProfile(path)/$f/$mhProfile(mh-sequences)
+	    }
 	    return
 	}
     }
@@ -551,6 +621,10 @@ proc Mh_Refile {srcFolder msgs folder} {
 	eval {MhExec refile} $chunk {-src +$srcFolder +$folder}
     }
 }
+proc Mh_RefileFile {folder file} {
+    Exmh_Debug exec refile -link -file $file +$folder
+    eval {exec refile -link -file $file +$folder}
+}
 proc Mh_Copy {srcFolder msgs folder} {
     while {[llength $msgs] > 0} {
 	set chunk [lrange $msgs 0 19]
