@@ -65,6 +65,13 @@ proc Mime_Init {} {
 	}
     }
 
+    # Make a try to load Img extention, ignore failures
+    if [ catch { set img_version [package require Img] } ] {
+	Exmh_Debug Unable to load Img
+    } else {
+	 Exmh_Debug Loaded Img version $img_version
+    }
+
     set imageFilters [concat [option get . imageFilters {}] \
 			      [option get . imageUFilters {}]]
     foreach imageFilter $imageFilters {
@@ -176,6 +183,19 @@ will be translated into a suitable DOS 8.3 representation."}
 	{mime(mdnTo) dispositionNotificationTo {} {Disposition Notification To}
 "The address where you want MDN receipts to end up.  This normally your
 own email address."}
+	{mime(mdnSend) dispositionNotificationSend
+         {CHOICE never deny {ask user} auto/ask auto/ignore } 
+         {Send MDNs}
+"This controls if and how to send MDNs (receipts).  The options are:
+  - Never generate MDN, always ignore requests silently.
+  - Automatically generate a \"denied\" MDN when a MDN is requested.
+  - Ask user whether to generate a MDN when displaying the message.
+  - Automatically when displaying the message, but ask user in 
+    certain cases where it may not be appropriate to generate MDNs.
+  - As above, but silently ignore potentially suspect MDN requests."}
+	{mime(explainReport) explainReports OFF {Show report explanations}
+"This controls whether human-readable explanations are presented along
+with the machine-readable part (part 2) of multipart/report messages."}
     }
     set i 0
     foreach char {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z \
@@ -253,6 +273,13 @@ proc MimeColor { tkw color } {
 # message/rfc822 only:
 # mimeHdr($part,fullHeaders)	Boolean to display full headers
 #
+# multipart/report part 2 only:
+# mimeHdr($part,explainReport)  Display explanation of machine-readable part
+#                               of report.
+# mimeHdr($part,originalMessageDate)    Date of reported message, if known.
+# mimeHdr($part,originalMessageEnclosed)   Indicates whether the reported
+#                               message is enclosed as part 3 of multipart.
+#
 # images only:
 # mimeHdr($part,photo)		The image itself
 
@@ -279,7 +306,7 @@ proc MimeHeader {part contentType encoding} {
 	    set key [string trim [string tolower $key]]
 	    set val [string trim $val]
 	    # Allow single as well as double quotes
-	    if [regexp {^["']} $val quote] {
+	    if [regexp {^[\"']} $val quote] {
 		if [regexp ^${quote}(\[^$quote\]+)$quote $val x val2] {
 		    # Trim quotes and any extra crap after close quote
 		    set val $val2
@@ -338,16 +365,16 @@ proc MimeSetCharset {tkw part} {
 	set mime(fontSize) [string trim $mime(fontSize)]
 	if [catch {Mime_GetFont $tkw medium r plain $mime(fontSize) $charset} \
 		font] {
-	    MimeInsertNote $tkw [MimeLabel $part part] "Error: $font"
+	    MimeInsertNote $tkw [MimeLabel $part charset] "Error: $font"
 	    $tkw insert insert \n
 	    set mimeFont(charset,$charset) $mimeFont(default)
 	} else {
 	    set mimeFont(charset,$charset) $font
 	}
     }
-    set partTag [MimeLabel $part part]
+    set partTag [MimeLabel $part charset]
     if [catch {$tkw tag configure $partTag -font $mimeFont(charset,$charset)} err] {
-	MimeInsertNote $tkw [MimeLabel $part part] "Error: $err"
+	MimeInsertNote $tkw [MimeLabel $part charset] "Error: $err"
 	$tkw insert insert \n
 	Exmh_Status "No good font for $charset character set"
 	set mimeFont(charset,$charset) $mimeFont(default)
@@ -446,6 +473,10 @@ proc MimeWithDisplayHiding {tkw part body} {
 	    -variable mimeHdr($part,display)
     if {$mimeHdr($part,display)} {
 	uplevel $body
+    } else {
+        $tkw insert insert ". . .\t\t\t\t\t\t"
+        MimeInsertNote $tkw [MimeLabel $part part] \
+                "Invoke menu with right button."
     }
 }
 proc MimeSetPartVars {descVar displayedPartVar tkw part partTag} {
@@ -546,7 +577,7 @@ proc MimeSizeDialog {t size} {
     set f [frame $g.pad -bd 20]
     set msg [Widget_Message $f msg -text \
 	"Very Large Message: $size bytes" -width 200]
-    Widget_AddBut $f stop STOP {set mime(stop) 1} {top padx 2 pady 2}
+    Widget_AddBut $f stop STOP {set mime(stop) 1} {top padx 2 pady 2 filly}
     bind $f.stop <Any-Key> {set mime(stop) 1; Exmh_Status Stop warn}
     pack $f
     Widget_PlaceDialog $t $g
@@ -579,9 +610,24 @@ proc MimeSetDisplayFlag {part} {
 	if [regexp {^multipart} $mimeHdr($part,type)] {
 	    set mimeHdr($part,display) \
 		[expr ![regexp {parallel$} $mimeHdr($part,type)]]
+	    return
 	} elseif [regexp {^image} $mimeHdr($part,type)] {
 	    set mimeHdr($part,display) $mime(showImage)
-	} elseif ![info exists mimeHdr($part,display)] {
+	    return
+	}
+	if [regexp {^text.*vcard} $mimeHdr($part,type)] {
+	    set mimeHdr($part,display) 0
+	    Exmh_Debug "vcard: not displaying"
+	    return
+	}
+	if {[info exists mimeHdr($part,hdr,content-disposition)]} {
+	    set disp [split $mimeHdr($part,hdr,content-disposition) \;]
+	    if [regexp {ancillary} [lindex $disp 0]] {
+		set mimeHdr($part,display) 0
+		return
+	    }
+	}
+	if ![info exists mimeHdr($part,display)] {
 	    set mimeHdr($part,display) \
 		[expr {[file exists $mimeHdr($part,file)] && \
 			([file size $mimeHdr($part,file)] < $msg(maxsize))}]
@@ -678,6 +724,10 @@ proc MimeRedisplayPart {tkw part} {
 	    $triangle create line 0 5 10 5 -arrow last 
 	}
 	MimeShowPartBody $tkw $part
+
+	if [$tkw compare insert != "insert linestart"] {
+	    $tkw insert insert "\n"
+	}
     }
 
     MimeCleanTag $tkw 1
@@ -900,15 +950,22 @@ proc MimeMetaMail {contentType encoding fileName} {
 }
 
 proc Mime_ShowText {tkw part} {
-    global mimeHdr mime miscRE
+    global mimeHdr mime miscRE pgp
 
     MimeWithDisplayHiding $tkw $part {
 	set subtype [file tail $mimeHdr($part,type)]
 	Mime_WithTextFile fileIO $tkw $part {
 	    gets $fileIO firstLine
-	    if [regexp $miscRE(beginpgp) $firstLine] {
+	    if {$pgp(enabled) && [regexp $miscRE(beginpgp) $firstLine]} {
+		# convert from text/plain to application/pgp
 		set mimeHdr($part,type) "application/pgp"
 		catch { unset mimeHdr($part,typeDescr) }
+                if {[info exists mimeHdr($part,param,format)]} {
+                    unset mimeHdr($part,param,format)
+                    set i [lsearch $mimeHdr($part,params) format]
+                    set mimeHdr($part,params) \
+                        [lreplace $mimeHdr($part,params) $i $i]
+                }
 		Pgp_ShowMessage $tkw $part
 	    } else {
 		$tkw insert insert "$firstLine\n"
@@ -941,10 +998,27 @@ proc Mime_WithTextFile {fileIOVar tkw part body} {
 	return 1
     }
 
+    # Set the encoding for the file based on its character set
+
+    Mime_SetFileEncoding $fileIO $part
+
     uplevel $body
 
     MimeClose $fileIO
 }
+
+proc Mime_SetFileEncoding {fileIO part} {
+    global tcl_version
+    if {$tcl_version >= 8.1} {
+	set charset [Mime_GetCharset {} $part]
+	regsub {^us-} $charset {} charset
+	regsub {^(mac|iso)-} $charset {\1} charset
+	if {[catch {fconfigure $fileIO -encoding $charset} err]} {
+	    Exmh_Debug "Mime_SetFileEncoding: $err"
+	}
+    }
+}
+
 proc Mime_GetUnencodedFile {part} {
     global mimeHdr mime
 
@@ -982,11 +1056,16 @@ proc Mime_GetCharset {tkw part} {
     set charset us-ascii
     if [info exists mimeHdr($part,param,charset)] {
 	set charset [string tolower $mimeHdr($part,param,charset)]
-	if ![info exists mime(registry,$charset)] {
-	    MimeInsertNote $tkw [MimeLabel $part part] \
-			   "Unknown charset: <$charset>"
-	    $tkw insert insert \n
-	    set charset us-ascii
+	if {[string length $tkw]} {
+
+	    # Limit the encodings to ones for which we have fonts.
+
+	    if ![info exists mime(registry,$charset)] {
+		MimeInsertNote $tkw [MimeLabel $part charset] \
+			       "Unknown charset: <$charset>"
+		$tkw insert insert \n
+		set charset us-ascii
+	    }
 	}
     }
     return $charset
@@ -1004,14 +1083,8 @@ proc Mime_ShowRfc822 {tkw part} {
 	}
 	MimeShowHeaders $tkw $part=1 [MimeLabel $part part]
 	MimeInsertSeparator $tkw $part 6
-    	if {$part == 0 \
-	    && ![info exists mimeHdr(0=1,hdr,x-exmhmdn)] \
-	    && [info exists mimeHdr(0=1,hdr,disposition-notification-to)]} {
-	    if [info exists mime(mdnDone)] {
-		unset mime(mdnDone)
-	    } else {
-		MDNAsk $tkw $mimeHdr(0=1,hdr,disposition-notification-to)
-	    }
+    	if {$part == 0} {
+	    MDNCheck $tkw
         }
 
 	MimeShowPart $tkw $part=1 [MimeLabel $part part] 1
@@ -1021,9 +1094,85 @@ proc Mime_ShowRfc822 {tkw part} {
 		-label "Show full message headers" \
 		-command [list busy MimeRedisplayHeaders $tkw $part=1] \
 		-variable mimeHdr($part=1,fullHeaders)
+	MimeMenuAdd $part command \
+		-label "Extract a message/rfc822 into current folder" \
+		-command [list busy MimeExtractMessage $tkw $part]
     }
     return 1
 }
+
+proc MimeExtractMessage {tkw part} {
+    global exmh mimeHdr
+
+    Mh_RefileFile $exmh(folder) $mimeHdr($part,file)
+    Scan_Folder $exmh(folder) 0
+    Exmh_Status "Extraction of part $part...done" blue
+}
+
+proc Mime_ShowMDN {tkw part} {
+    Mime_ShowReport $tkw $part report { Mime_ExplainMDN $tkw $part report }
+}
+
+proc Mime_ShowDSN {tkw part} {
+    Mime_ShowReport $tkw $part report { Mime_ExplainDSN $tkw $part report }
+}
+
+proc Mime_ShowReport {tkw part reportVar body } {
+    global mimeHdr mime
+    upvar $reportVar report
+
+    if ![info exists mimeHdr($part,explainReport)] {
+	set mimeHdr($part,explainReport) $mime(explainReport)
+    }
+
+    if {[string compare $part "0"] != 0} {
+	MimeMenuAdd $part checkbutton \
+		-label "Show explanation" \
+		-command [list busy MimeRedisplayPart $tkw $part] \
+		-variable mimeHdr($part,explainReport)
+    }
+
+    if [Mime_ShowText $tkw $part] {
+	if { $mimeHdr($part,explainReport) && $mimeHdr($part,display) } {
+	    MimeInsertSeparator $tkw $part 6
+
+	    ParseReport $part report
+
+	    if [info exists report(original-recipient)] {
+		set recipient $report(original-recipient)
+	    } elseif [info exists report(final-recipient)] {
+		set recipient $report(final-recipient)
+	    } else {
+		set recipient "(unknown)"
+	    }
+
+	    $tkw insert insert \
+	        "This is a report concerning the message you sent\
+		 \n  to:      $recipient"
+
+	    if [info exists mimeHdr($part,originalMessageDate)] {
+		$tkw insert insert \
+		"\n  on date: $mimeHdr($part,originalMessageDate)"
+	    }
+
+	    if [info exists mimeHdr($part,originalMessageEnclosed)] {
+		if { $mimeHdr($part,originalMessageEnclosed) } {
+		    $tkw insert insert "\n(A copy of the\
+                        message is included below.)"
+		}
+	    }
+
+	    $tkw insert insert "\n\n"
+
+	    uplevel $body
+	} else {
+	    return 1
+	}
+    } else {
+	return 0
+    }
+}
+
 proc MimeShowHeaders {tkw part overTag} {
     global mimeHdr
 
@@ -1057,6 +1206,9 @@ proc MimeShowFullHeaders {tkw part inlin} {
 	    set tag [Mime_PrintEncodedHeader $tkw [MimeLabel $part headers] \
 				    $mimeHdr($part,hdr,$hdr) \
 				    medium r plain $mime(fontSize)]
+	    if [regexp -nocase date $truehdr] {
+		MimeShowTime $tkw $mimeHdr($part,hdr,$hdr)
+	    }
 	    $tkw insert insert \n
 	    foreach key [list $truehdr default] {
 		if [info exists msg(tag,$key)] {
@@ -1070,8 +1222,36 @@ proc MimeShowFullHeaders {tkw part inlin} {
 	}
     }
 }
+
+# Show local time if message from another tz
+# e.g.
+#      Mon, 18 May 1998 11:58:39 GMT
+# is shown as
+#      Mon, 18 May 1998 11:58:39 GMT (13:58 MET DST)
+#
+
+proc MimeShowTime { tkw time } {
+    global mime
+
+    catch {
+	set msgtime   [clock scan $time]
+	set localtime [clock format $msgtime -format " %T"]
+	if { [string first $localtime $time] == -1 } {
+	    Preferences_Resource mime(localTimeFormat) localTimeFormat \
+		"%H:%M %Z"
+	    set format $mime(localTimeFormat)
+	    set day [clock format $msgtime -format %a]
+	    if { [string first $day $time] == -1 } {
+		set format "%a $format"
+	    }
+	    set msgtime [clock format $msgtime -format $format]
+	    $tkw insert insert " ($msgtime)"
+	}
+    }
+}
+
 proc MimeShowMinHeaders {tkw part inlin} {
-    global mimeHdr mhProfile msg mime tk_version
+    global mimeHdr mhProfile msg mime
     
     if ![info exists mimeHdr($part,hdrs)] {
 	return
@@ -1112,6 +1292,9 @@ proc MimeShowMinHeaders {tkw part inlin} {
 	set tag [Mime_PrintEncodedHeader $tkw [MimeLabel $part headers] \
 				$mimeHdr($part,hdr,$hdr) \
 				medium r plain $mime(fontSize)]
+	if [regexp -nocase date $truehdr] {
+	    MimeShowTime $tkw $mimeHdr($part,hdr,$hdr)
+	}
 	$tkw insert insert \n
 	foreach key [list $truehdr [expr {$show ? "default" : "hidden"}]] {
 	    if [info exists msg(tag,$key)] {
@@ -1124,11 +1307,7 @@ proc MimeShowMinHeaders {tkw part inlin} {
 	}
 	if {! $show} {
 	    set hideMark [$tkw index insert]
-	    if {$tk_version >= 4.0} {
-		$tkw mark set insert "end -1c"
-	    } else {
-		$tkw mark set insert end
-	    }
+	    $tkw mark set insert "end -1c"
 	}
     }
     if {$inlin} {
@@ -1395,7 +1574,7 @@ proc Mime_ShowMultipart {tkw part} {
 	    frame $g -bd 4 -relief raised
 	    set f [frame $g.pad -bd 20]
 	    set msg [Widget_Message $f msg -text "$numParts Parts" -width 200]
-	    Widget_AddBut $f stop STOP {set mime(stop) 1} {top padx 2 pady 2}
+	    Widget_AddBut $f stop STOP {set mime(stop) 1} {top padx 2 pady 2 filly}
 	    bind $f.stop <Any-Key> {set mime(stop) 1; Exmh_Status Stop warn}
 	    pack $f
 	    Widget_PlaceDialog $exwin(mtext) $g
@@ -1532,6 +1711,24 @@ proc Mime_ShowMultipartAlternative {tkw part} {
 		  -variable mimeHdr($part,chosenPart) \
 		  -value $subpart
     }
+
+    # If we find a type we like, use that subpart instead
+    # This is to support display of text/plain instead of text/html
+
+    set preferred [option get . mime_alternative_prefs {}]
+    set selected 0
+    foreach preftype $preferred {
+	for {set subpart 1} {$subpart <= $numParts} {incr subpart} {
+	    set type $mimeHdr($part=$subpart,type)
+	    if {$type == $preftype} {
+		set mimeHdr($part,chosenPart) $subpart
+		set selected 1
+		break
+	    }
+	}
+	if {$selected != 0} {break}
+    }
+
     set mimeHdr($part,priorChosenPart) \
 	$mimeHdr($part,chosenPart)
 
@@ -1544,71 +1741,165 @@ proc Mime_ShowMultipartAlternative {tkw part} {
 
     return 1
 }
+
+proc MimeShowReportPart {tkw part overTag only reportPart} {
+    global mimeHdr
+
+    set partTag [MimeLabel $part part]
+    if ![info exists mimeHdr($part,decode)] {
+	# decode sub-parts by default, but not main body
+	set mimeHdr($part,decode) [string compare $part 0=1]
+    }
+    MimeWithTagging $tkw $partTag $overTag \
+		    {-background $mimeHdr($part,color) \
+		     -foreground [$tkw cget -foreground]} {
+	MimeSetPartVars desc displayedPart $tkw $part $partTag
+	MimeSetStdMenuItems $tkw $part
+	MimeShowPartHeader $tkw $part $partTag $only $reportPart $desc
+	MimeShowPartBody $tkw $part
+    }
+    Exmh_Status $desc
+}
+
 proc Mime_ShowMultipartReport {tkw part} {
     global mimeHdr mime
 
-    if ![info exists mimeHdr($part,param,boundary)] {
-	$tkw insert insert "No <boundary> parameter for multipart message\n"
-	$tkw insert insert "Raw content follows...\n\n"
-	return [Mime_ShowText $tkw $part]
-    }
-    set numParts $mimeHdr($part,numParts)
-    if { $numParts > 3 || $numParts < 2} {
-	$tkw insert insert "Incorrect number of parts in multipart/report\n"
-	$tkw insert insert "Raw content follows...\n\n"
-	return [Mime_ShowText $tkw $part]
-    }
-    if {![info exists mimeHdr($part,param,report-type)]} {
-	return [Mime_ShowMultipart $tkw $part]
-    }
-    switch $mimeHdr($part,param,report-type) {
-	"disposition-notification" {
-	    return [Mime_ShowMultipartReportMDN $tkw $part]
+    MimeWithDisplayHiding $tkw $part {
+	if ![info exists mimeHdr($part,param,boundary)] {
+	    $tkw insert insert "No <boundary> parameter for multipart message\n"
+	    $tkw insert insert "Raw content follows...\n\n"
+	    return [Mime_ShowText $tkw $part]
 	}
-	default {
-	    # Fallback for report types we don't handle yet
-	    return [Mime_ShowMultipart $tkw $part]
+
+	set numParts $mimeHdr($part,numParts)
+
+	if { $numParts < 2 || $numParts > 3 } {
+	    $tkw insert insert "Incorrect number of parts in multipart/report\n"
+	}
+
+	if { $numParts >= 1 } {
+	    set mimeHdr($part=1,color) \
+		[MimeDarkerColor $tkw $mimeHdr($part,color)]
+	    set mimeHdr($part=1,justDoIt) 1
+	    MimeShowReportPart $tkw $part=1 [MimeLabel $part part] 0 \
+		"Human-readable report  "
+	}
+
+	if { $numParts >= 2 } {
+	    set mimeHdr($part=2,color) \
+		[MimeDarkerColor $tkw $mimeHdr($part,color)]
+	    set mimeHdr($part=2,justDoIt) 1
+	    MimeInsertSeparator $tkw $part 8
+
+	    if { $numParts >= 3 } {
+		set mimeHdr($part=2,originalMessageEnclosed) 1
+
+		if [catch {open $mimeHdr($part=3,file)} in] {
+		    error "Cannot read original message file"
+		} else {
+		    while {! [eof $in]} {
+			set len [gets $in line]
+			if {[string match "date:*" [string tolower $line]]} {
+			    set mimeHdr($part=2,originalMessageDate) \
+				[string trim [string range $line 5 end]]
+			}
+		    }
+		    close $in
+		}
+
+	    } else {
+		set mimeHdr($part=2,originalMessageEnclosed) 0
+	    }
+
+	    MimeShowReportPart $tkw $part=2 [MimeLabel $part part] 0 \
+		"Machine-readable report"
+	}
+	
+	if { $numParts >= 3 } {
+	    set mimeHdr($part=3,color) \
+		[MimeDarkerColor $tkw $mimeHdr($part,color)]
+	    set mimeHdr($part=3,justDoIt) 1
+	    MimeInsertSeparator $tkw $part 8
+	    MimeShowReportPart $tkw $part=3 [MimeLabel $part part] 0 \
+		"Original message       "
+	}
+
+	for {set subpart 4} {$subpart <= $numParts} {incr subpart} {
+	    set mimeHdr($part=$subpart,color) \
+		[MimeDarkerColor $tkw $mimeHdr($part,color)]
+	    set mimeHdr($part=$subpart,justDoIt) 1
+	    MimeInsertSeparator $tkw $part 8
+	    MimeShowPart $tkw $part=$subpart [MimeLabel $part part] 0
 	}
     }
 }
-proc Mime_ShowMultipartReportMDN {tkw part} {
-    global mimeHdr mime
-    set numParts $mimeHdr($part,numParts)
-    set from "unknown"
-    set date "010170"
-    if [info exists mime(mdnDone)] {
-	# Magic button has been pressed, fallback
-	unset mime(mdnDone)
-	return [Mime_ShowMultipart $tkw $part]
+
+proc ParseReport {part reportVar} {
+
+    global mimeHdr
+
+    upvar $reportVar report
+    set uniq 1
+    set cur  " "
+
+    if [catch {open $mimeHdr($part,file)} in] {
+	error "Cannot read report body"
     }
-    if [catch {open $mimeHdr($part=2,file)} in] {
-	error "Cannot read disposition file"
-    }
-    for {set len [gets $in line]} {! [eof $in]} {set len [gets $in line]} {
-	if {[string match "final-recipient:*" [string tolower $line]]} {
-	    if { [scan [string trim [string range $line 15 end]] \
-			"%\[^\;]%c%s" * * from] != 3 } {
-		set from [string trim [string range $line 15 end]]
+    while {! [eof $in]} {
+	gets $in line
+Exmh_Status "Parse report: $line"
+	if ![regexp {^[	 ]} $line] {
+	    if [regexp -indices {^([^:]+):} $line match hdr] {
+		set cur [string tolower [eval {string range $line} $hdr]]
+		if [info exists report($cur)] {
+		    # Duplicate header
+		    set cur :$uniq:$cur
+		    incr uniq
+		}
+		set report($cur) \
+		    [string trim \
+			 [string range $line \
+			      [expr [lindex $match 1]+1] end]]
+Exmh_Status "  header: $cur: $report($cur)"
 	    }
-	    set from [string trim $from \>\<]
-	} elseif {[string match "disposition:*" [string tolower $line]]} {
-	    set disposition [string trim [string range $line 12 end]]
+	} elseif [regexp -indices {^[	 ]+} $line match] {
+	    append report($cur) \n$line
 	}
     }
     close $in
-    if ![catch {open $mimeHdr($part=3,file)} in] {
-	for {set len [gets $in line]} {! [eof $in]} \
-		{set len [gets $in line]} {
-	    if {[string match "date:*" [string tolower $line]]} {
-		set date [string trim [string range $line 5 end]]
-	    }
-	}
-	close $in
-    }
+}
 
-    MDNReportDialog $tkw $from $date $disposition $numParts
+proc Mime_ExplainMDN {tkw part reportVar} {
+    global mimeHdr mime
+    upvar $reportVar report
+
+    if [info exists report(disposition)] {
+	$tkw insert insert \
+"This report is a message disposition notification, which reports
+details on how the recipient received the message.\n\n"
+	MDNExplainDisposition $tkw report
+    } else {
+	$tkw insert insert \
+"This report is a message disposition notification, which indicates that
+the message probably was received by the recipient, but no further details 
+are known.  There is no guarantee that the message actually was seen
+or acted on by the recipient."
+    }
     return 1
 }
+
+proc Mime_ExplainDSN {tkw part reportVar} {
+    global mimeHdr mime
+    upvar $reportVar report
+
+    $tkw insert insert "\
+This report is a delivery status notification, that reports actions taken
+by the message transport system in delivering the message to the 
+recipient."
+
+    return 1
+}
+
 proc MimeChopPart {tkw part} {
     # Chop up the parts at this level if it hasn't already been done.
     global mimeHdr
@@ -1714,7 +2005,7 @@ proc MimeParseMulti {tkw part fileIO boundary defType} {
     return $subpart
 }
 proc MimeParseSingle {tkw part fileIO } {
-    global mimeHdr mime miscRE tk_version msg
+    global mimeHdr mime miscRE msg
 
     set mimeHdr($part=1,color) $mimeHdr($part,color)
     set part $part=1
@@ -1795,18 +2086,12 @@ proc MimeParseSingle {tkw part fileIO } {
 		MimeShowMinHeaders $tkw $part 1
 	    }
 	    MimeInsertSeparator $tkw $part 6
-	    if {![info exists mimeHdr(0=1,hdr,x-exmhmdn)] && \
-		   [info exists mimeHdr(0=1,hdr,disposition-notification-to)]} {
-		if [info exists mime(mdnDone)] {
-		    unset mime(mdnDone)
-		} else {
-		    MDNAsk $tkw $mimeHdr(0=1,hdr,disposition-notification-to)
-		}
-	    }
+	    MDNCheck $tkw
 	    if [info exists mimeHdr($part,param,charset)] {
 		set tag [MimeSetCharset $tkw $part]
 		$tkw tag remove noteTag "insert -1line"  end
 		$tkw tag add $tag insert end
+		Mime_SetFileEncoding $fileIO $part
 	    }  else {
 		set tag {}
 	    }
@@ -1896,10 +2181,7 @@ proc MimeMakeMenu {tkw tag part menuLabel} {
 	}
     }
 
-    global tk_version
-    if {$tk_version >= 4.0} {
-	$mimeHdr($part,menu) config -tearoff 0
-    }   
+    $mimeHdr($part,menu) config -tearoff 0
     $mimeHdr($part,menu) configure -disabledforeground Black
     MimeMenuAdd $part command \
 	-label $menuLabel \
