@@ -8,6 +8,78 @@
 # todo:
 
 # $Log: pgpMisc.tcl,v $
+# Revision 1.21  1999/08/25 15:42:23  bmah
+# exmh now times out passphrases for PGP subkeys correctly.
+#
+# Revision 1.20  1999/08/22 18:17:08  bmah
+# Email PGP queries now go out correctly.  Use Exmh_Status to inform
+# user of state of an outgoing email key query.
+#
+# Revision 1.19  1999/08/13 00:39:05  bmah
+# Fix a number of key/passphrase management problems:  pgpsedit now
+# manages PGP versions, keys, and passphrases on a per-window
+# basis.  Decryption now works when no passphrases are cached.
+# One timeout parameter controls passphrases for all PGP
+# versions.  seditpgp UI slightly modified.
+#
+# Revision 1.18  1999/08/04 22:43:39  cwg
+# Got passphrase timeout to work yet again
+#
+# Revision 1.17  1999/08/04 19:50:44  cwg
+# Fixed problems with not providing a password under pgp2
+#
+# Revision 1.16  1999/08/04 16:30:18  cwg
+# Don't prompt for a passphrase when we shouldn't.
+#
+# Revision 1.15  1999/08/03 21:18:16  bmah
+# Fix a bug that caused signing without a password to silently fail
+# (instead of generate an error message).
+#
+# Revision 1.14  1999/08/03 17:19:51  bmah
+# Fix some procedures that were mis-named.  Also retrieve
+# Pgp_Misc_RemovePgpActionHeader from multipgp branch.
+#
+# Revision 1.13  1999/08/03 15:08:48  bmah
+# Check that pgp is enabled before attempting to check the version
+# used for a message.  Don't log PGP passphrase anymore.
+#
+# Revision 1.12  1999/08/03 04:05:55  bmah
+# Merge support for PGP2/PGP5/GPG from multipgp branch.
+#
+# Revision 1.11  1999/06/10 16:59:18  cwg
+# Re-enabled the timeout of PGP passwords
+#
+# Revision 1.10  1999/05/04 06:35:38  cwg
+# Fixed crash when aborting out of PGP Password window
+#
+# Revision 1.9  1999/04/30 20:41:55  cwg
+# get pgpPass(cur) in Misc_PostProcess
+#
+# Revision 1.8  1999/04/15 23:41:59  cwg
+# Make the crypt menu values be per-window instead of global.
+#
+# Revision 1.7  1999/04/10 04:20:08  cwg
+# Do the right thing if pgp(seditpgp) is not enabled.
+#
+# Revision 1.6  1999/04/09 19:11:26  cwg
+# Now that's an embarrasing typo
+#
+# Revision 1.5  1999/04/06 05:38:39  cwg
+# Bug fix.
+#
+# Revision 1.4  1999/04/04 20:34:57  cwg
+# Removed dead code which only ran in pre tk-4.1 versions.
+#
+# Revision 1.3  1999/04/01 15:38:10  cwg
+# Bug fix.  Wasn't working for people with PGP disabled.
+#
+# Revision 1.2  1999/03/26 08:41:55  cwg
+# Changes to PGP interface to use preferences variables instead of
+# message headers.  Also, reorganize the "PGP..." menu and rename it
+# "Crypt..."
+#
+# See the "PGP interface" preferences page for more info.
+#
 # Revision 1.1  1998/05/05 17:55:38  welch
 # Initial revision
 #
@@ -58,14 +130,14 @@
 #
 
 # creates the file and put the string in it
-proc Misc_StringFile { str filename } {
+proc Pgp_Misc_StringFile { str filename } {
     set file [open $filename w 0600]
     puts -nonewline $file $str
     close $file
 }
 
 # returns a string containing the whole file's content
-proc Misc_FileString { filename } {
+proc Pgp_Misc_FileString { filename } {
     set file [open $filename r]
     set result [read $file]
     close $file
@@ -73,9 +145,9 @@ proc Misc_FileString { filename } {
 }
 
 # unsign a pgp clearsigned message (take the pgp stuff out)
-proc Pgp_Unsign { text } {
+proc Pgp_Misc_Unsign { text } {
     if {![regexp "^(.*\n)?-+BEGIN PGP SIGNED\[^\n]*\n(Hash:\[^\n]*\n)?\n(.*)\n-+BEGIN PGP SIGNATURE" $text {} {} {} text]} {
-#	error "<Misc_Unsign> can't find the message"
+#	error "<Pgp_Misc_Unsign> can't find the message"
 # it is inconvenient and probably wrong to eat the message
 	return "Error in stripping PGP armor:\n$text"
     }
@@ -85,7 +157,7 @@ proc Pgp_Unsign { text } {
 }
 
 # returns a list of integers from $x to $y-1
-proc Misc_IntList { x y } {
+proc Pgp_Misc_IntList { x y } {
     for {set result {}} {$y > $x} {set x [expr $x + 1]} {
 	lappend result $x
     }
@@ -94,7 +166,7 @@ proc Misc_IntList { x y } {
 
 # returns a list containing every element of $list that fulfills
 # the requirement of cond(var)
-proc Misc_Filter { var cond list } {
+proc Pgp_Misc_Filter { var cond list } {
     upvar $var elem
 
     set result {}
@@ -109,7 +181,7 @@ proc Misc_Filter { var cond list } {
 
 # like filter, but returns a list of 2 lists. The first containing
 # the elements fulfilling the requirement, the second those that don't
-proc Misc_Segregate { var cond list } {
+proc Pgp_Misc_Segregate { var cond list } {
     upvar $var elem
 
     set fulfill {}
@@ -126,7 +198,7 @@ proc Misc_Segregate { var cond list } {
 }
 
 # applies expr on each elem and returns the resulting list
-proc Misc_Map { var expr list } {
+proc Pgp_Misc_Map { var expr list } {
     upvar $var elem
 
     set result {}
@@ -138,7 +210,7 @@ proc Misc_Map { var expr list } {
 }
 
 # returns the list reversed
-proc Misc_Reverse { list } {
+proc Pgp_Misc_Reverse { list } {
     set result {}
     foreach elem $list {
 	set result [linsert $result 0 $elem]
@@ -147,7 +219,7 @@ proc Misc_Reverse { list } {
 }
 
 # asks for a password in a window called $title with a little note $label
-proc Misc_GetPass { title label } {
+proc Pgp_Misc_GetPass { v title label } {
     global getpass pgp
     set w .getpass
 
@@ -176,42 +248,43 @@ proc Misc_GetPass { title label } {
     Widget_BindEntryCmd $getpass(entry) <Return> {
 	set getpass(state) "ok"
     }
-    global tk_version
-    if {$tk_version >= 4.0} {
-	# Override bindtags done by Widget_BindEntryCmd
-	bindtags $getpass(entry) $getpass(entry)
-    }
-    SeditBind $getpass(entry) backspace {
-	global getpass
-	if $pgp(echopass) {
-	    $getpass(entry) delete [expr [$getpass(entry) index end]-1] end
-	}
-	set getpass(pass) [string range $getpass(pass) 0 [expr [string length $getpass(pass)]-2]]
-    }
-    bind $getpass(entry) <Any-Key> {
-	global getpass
-	if {"%A" != "" && "%A" != "{}"} {
-	    if $pgp(echopass) {
-		$getpass(entry) insert insert "*"
-	    }
-	    append getpass(pass) "%A"
-	}
-    }
+    # Override bindtags done by Widget_BindEntryCmd
+    bindtags $getpass(entry) $getpass(entry)
+
+    SeditBind $getpass(entry) backspace "
+        global getpass
+        if \[set pgp($v,echopass)] \{
+            \$getpass(entry) delete \[expr \[\$getpass(entry) index end]-1] end
+        \}
+	set getpass(pass) \[string range \$getpass(pass) 0 \[expr \[string length \$getpass(pass)]-2]]
+    "
+
+    bind $getpass(entry) <Any-Key> "
+        global getpass
+        if \{ \"%A\" != \"\" && \"%A\" != \"\{\}\"\} \{
+            if \[set pgp($v,echopass)] \{
+                \$getpass(entry) insert insert \"*\"
+            \}
+            append getpass(pass) \"%A\"
+        \}
+    "
+
     bind $getpass(entry) <Control-U> {
 	global getpass
 	$w.pass.entry delete 0 end
 	set getpass(pass) {}
     }
+
     Visibility_Wait $w
     update idletasks
-    if $pgp(grabfocus)  {
+    if [set pgp($v,grabfocus)] {
 	grab -global $w
     }
     focus $getpass(entry)
     tkwait variable getpass(state)
     Exmh_Focus
     # Forget window locaton  to avoid problems in virtual root window managers
-    destroy $w	
+    destroy $w
     set password $getpass(pass)
     unset getpass(pass)
     
@@ -243,7 +316,7 @@ proc Misc_DisplayText { title text {height 8}} {
 
 # gets the whole header of the draft. Returns a list of strings.
 # Each multi-line header is put back into a single string (with embedded \n)
-proc Misc_GetHeader { in } {
+proc Pgp_Misc_GetHeader { in } {
     global miscRE
 
     set headers {}
@@ -265,29 +338,62 @@ proc Misc_GetHeader { in } {
 
 #
 proc Misc_PostProcess { srcfile } {
-    global mhProfile
+    global mhProfile pgp
+
+    set id [SeditId $srcfile]
 
     set dstfile [Mh_Path $mhProfile(draft-folder) new]
     set curfile $srcfile
 
     # read the header to see what postprocessing has to be done
     set in [open $curfile r]
-    set mailheader [Misc_GetHeader $in]
+    set mailheader [Pgp_Misc_GetHeader $in]
     close $in
 
     # call the pgp postprocesing if necessary
-    set hdrIndex [lsearch -glob $mailheader {[Pp]gp-[Aa]ction:*}]
-    if {($hdrIndex >= 0) &&
-            ![regexp -nocase {^[^ ]*: *none} [lindex $mailheader $hdrIndex]]} {
-	Pgp_Process $curfile $dstfile
-	set curfile $dstfile
+    # (checking for $pgp(version,$id) is necessary because
+    # drafts generated by exmh for PGP key retrieval will not
+    # have their own PGP variables set)
+    if {$pgp(enabled) && [info exists pgp(version,$id)]} {
+	if {$pgp(encrypt,$id) || $pgp(sign,$id)} {
+	    set v $pgp(version,$id)
+	    set keyid [lindex $pgp($v,myname,$id) 0]
+	    set subkeyid [lindex $pgp($v,myname,$id) 2]
+
+	    # If there's a passphrase from sedit and it's non-empty, use it
+	    if {$pgp(seditpgp)} {
+		if {[info exists pgp(cur,pass,$id)] && ([string length $pgp(cur,pass,$id)] > 0) && [info exists pgp($v,myname,$id)]} {
+		    set pgp($v,pass,$keyid) $pgp(cur,pass,$id)
+		    Pgp_SetPassTimeout cur $id
+		} else {
+		    set pgp($v,pass,$keyid) ""
+		}
+	    } else {
+		set pgp($v,pass,$keyid) [Pgp_GetPass $v $pgp($pgp(version,$id),myname,$id)]
+	    } 
+
+	    # Set timeouts
+	    Pgp_SetPassTimeout $pgp(version,$id) $keyid
+	    # Because of DecryptExpect we need to store passphrase
+	    # for mainkey and subkey
+	    if {[string length $subkeyid] > 0} {
+		Pgp_SetPassTimeout $pgp(version,$id) $subkeyid
+	    }
+
+# Danger Wil Robinson!
+	    #Exmh_Debug pass=>$pgp($v,pass,$keyid)<
+	    if {([info exists pgp($v,pass,$keyid)]) && ([string length $pgp($v,pass,$keyid)] != 0)} {
+		Pgp_Process $pgp(version,$id) $curfile $dstfile
+		set curfile $dstfile
+	    }
+	}
     }
 
     return [file tail $curfile]
 }
 
 #
-proc Misc_Send { to subject {bodyfile {}} {headers {}} } {
+proc Pgp_Misc_Send { to subject {bodyfile {}} {headers {}} } {
     global mhProfile
 
     MhExec comp -nowhatnowproc
@@ -295,7 +401,7 @@ proc Misc_Send { to subject {bodyfile {}} {headers {}} } {
 
     # read the default mail header
     set in [open "$mhProfile(path)/$mhProfile(draft-folder)/$msg" r]
-    set header [Misc_GetHeader $in]
+    set header [Pgp_Misc_GetHeader $in]
     close $in
 
     # write it back with a new to and subject fields
@@ -317,7 +423,7 @@ proc Misc_Send { to subject {bodyfile {}} {headers {}} } {
 
     Mh_Send $msg
 }
-proc Pgp_FixHeader { line } {
+proc Pgp_Misc_FixHeader { line } {
     if [regexp {^([^:]+):(.*)} $line x key value] {
 	set newline {}
 	while {[regexp {([^-]+)-(.*)} $key  x first rest]} {
@@ -332,3 +438,28 @@ proc Pgp_FixHeader { line } {
 	return $line
     }
 }
+# This was part of SeditEncrypt
+proc Pgp_Misc_RemovePgpActionHeader { t varHasfcc } {
+    global miscRE
+    upvar $varHasfcc hasfcc
+
+    set linenb 1
+    set line [$t get $linenb.0 $linenb.end]
+
+    set hasfcc 0
+    while {![regexp $miscRE(headerend) $line]} {
+	if [regexp -nocase {^pgp-action:} $line] {
+	    set line " dummy"
+	    while {[regexp "^\[ \t]" $line]} {
+		$t delete $linenb.0 [expr {$linenb + 1}].0
+		set line [$t get $linenb.0 $linenb.end]
+	    }
+	} else {
+	    if [regexp -nocase {^fcc:} $line] {
+		set hasfcc 1
+	    }
+	    set linenb [expr {$linenb + 1}]
+	}
+	set line [$t get $linenb.0 $linenb.end]
+    }
+}
