@@ -41,12 +41,112 @@ proc Preferences_Init { userDefaults appDefaults } {
     set pref(localDefaults) \
 	"[file dirname $appDefaults]/local.[file tail $appDefaults]"
 
+    catch {exec mkdir -p [glob ~]/.exmh}
+
+    if {[file exists [glob ~]/.exmh-defaults] 
+	&& ![file exists $userDefaults]} { 
+	PreferencesCopyToNewLocation
+    }
+    
     PreferencesReadFile $pref(appDefaults) startup
     PreferencesReadFile $pref(localDefaults) 50
     PreferencesReadFile $pref(userDefaults) user
 
     Preferences_Resource pref(helpInOneWindow) helpInOneWindow 1
 }
+
+proc PreferencesCopyToNewLocation {} {
+    global pref
+
+    Widget_Toplevel .newprefs "Copy User Preferences"
+    Widget_Message .newprefs msg -aspect 1000 -text "
+It appears you have not run Exmh since the [file tail $pref(userDefaults)] 
+and other related files moved in to the ~/.exmh directory.
+
+Is it ok if Exmh copies the files to your ~/.exmh directory?  Once you 
+have verified that Exmh is functioning properly, you can delete the 
+old files by hand.
+"
+
+    Widget_Frame .newprefs rim Pad {top expand fill}
+    .newprefs.rim configure -bd 10
+   
+    Widget_Frame .newprefs.rim but Menubar {top fill}
+    Widget_AddBut .newprefs.rim.but yes "Yes" PreferencesDoCopyFiles
+    Widget_AddBut .newprefs.rim.but no "No, Exit" {destroy .newprefs ; exit }
+    tkwait window .newprefs
+}
+
+proc PreferencesCopyDone { movedFiles } {
+    Widget_Toplevel .filesmoved "Files Copied"
+    Widget_Message .filesmoved msg -aspect 1000 -text "
+The following files have been copied, and the copies have been renamed.  
+Once you have verified that Exmh is working properly, you can delete 
+these files.
+
+$movedFiles
+"
+
+    Widget_Frame .filesmoved rim Pad {top expand fill}
+    .filesmoved.rim configure -bd 10
+
+    Widget_Frame .filesmoved.rim but Menubar {top fill}
+    Widget_AddBut .filesmoved.rim.but ok "OK" {destroy .filesmoved }
+    tkwait window .filesmoved
+}
+
+proc PreferencesDoCopyFiles { } {
+    if {[file exists [glob ~]/.exmh-defaults] 
+	&& ![file exists [glob ~]/.exmh/exmh-defaults]} {
+	catch {exec cp -r [glob ~]/.exmh-defaults [glob ~]/.exmh/exmh-defaults}
+	append movedList ".exmh-defaults\n"
+    }
+	
+    if {[file exists [glob ~]/.exmh-defaults-mono]
+        && ![file exists [glob ~]/.exmh/exmh-defaults-mono]} {
+        catch {exec cp -r [glob ~]/.exmh-defaults-mono [glob ~]/.exmh/exmh-defaults-mono}
+        append movedList ".exmh-defaults-mono\n"
+    }
+
+    if {[file exists [glob ~]/.exmh-defaults-color]
+        && ![file exists [glob ~]/.exmh/exmh-defaults-color]} {
+        catch {exec cp -r [glob ~]/.exmh-defaults-color [glob ~]/.exmh/exmh-defaults-color}
+        append movedList ".exmh-defaults-color\n"
+    }
+
+    if {[file exists [glob ~]/.exmh_addrs] 
+	&& ![file exists [glob ~]/.exmh/exmh_addrs]} {
+	catch {exec cp -r [glob ~]/.exmh_addrs [glob ~]/.exmh/exmh_addrs}
+        append movedList ".exmh_addrs\n"
+    }
+
+    if {[file exists [glob ~]/.exmh_addrs.bak] 
+	&& ![file exists [glob ~]/.exmh/exmh_addrs.bak]} {
+	catch { exec cp -r [glob ~]/.exmh_addrs.bak [glob ~]/.exmh/exmh_addrs.bak}
+        append movedList ".exmh_addrs.bak\n"
+    }
+    
+    if {[file exists [glob ~]/.exmh-images] 
+	&& ![file exists [glob ~]/.exmh/exmh-images]} {
+	catch { exec cp -r [glob ~]/.exmh-images [glob ~]/.exmh/exmh-images}
+        append movedList ".exmh-images\n"
+    }
+    
+    if {[file exists [glob ~]/.exmhbindings] 
+	&& ![file exists [glob ~]/.exmh/exmhbindings]} {
+	catch { exec cp -r [glob ~]/.exmhbindings [glob ~]/.exmh/exmhbindings}
+        append movedList ".exmhbindings\n"
+    }
+    
+    if {[file exists [glob ~]/.exmhsedit] 
+	&& ![file exists [glob ~]/.exmh/exmhsedit]} {
+	catch { exec cp -r [glob ~]/.exmhsedit [glob ~]/.exmh/exmhsedit}
+        append movedList ".exmhsedit\n"
+    }
+    catch {destroy .newprefs}
+    PreferencesCopyDone "$movedList"
+}
+
 proc PreferencesReadFile { basename level } {
     if [file exists $basename] {
 	if [catch {option readfile $basename $level} err] {
@@ -54,7 +154,7 @@ proc PreferencesReadFile { basename level } {
 	}
     }
 
-    if {[tk colormodel .] == "color"} {
+    if {[winfo depth .] > 4} {
 	if [file exists $basename-color] {
 	    if [catch {option readfile $basename-color $level} err] {
 		Exmh_Status "Error in $basename-color: $err"
@@ -260,7 +360,6 @@ proc PreferencesSectionDialog { id } {
 	    PreferencesDialogItem $body $id $item $maxWidth
 	}
     }
-    Exwin_ToplevelFocus .pref$ix none
     set pref(label) $buttons.label
 }
 proc PreferencesNext { ix {i 1}} {
@@ -558,7 +657,7 @@ proc Preferences_Resource { _varName _rname _default } {
 }
 
 proc PreferencesPaneHelp { id {gotoxres {}} } {
-    global pref tk_version
+    global pref
     set ix [lsearch $pref(panes) $id]
     set top .prefhelplong$ix
     set t $top.t
@@ -580,14 +679,9 @@ proc PreferencesPaneHelp { id {gotoxres {}} } {
 	$t configure -state disabled
     }
 
-    if {$tk_version >= 4.0} {
-	# make as much help text visible
-	$t see end
-        $t see help4_$gotoxres
-    } else {
-	$t yview end
-        $t yview help4_$gotoxres
-    }
+    # make as much help text visible
+    $t see end
+    $t see help4_$gotoxres
 }
 proc PreferencesPaneHelpText { t item } {
     set res [PrefXres $item] 
