@@ -44,7 +44,7 @@ proc SeditSigfileDefault {} {
     return $propersig
 }
 proc Sedit_Start { draft } {
-    global sedit intelligentSign quote msg
+    global sedit intelligentSign quote msg pgp
     global exmh	;# for menu references to $exmh(...)
     if ![info exists sedit(init)] {
 	Sedit_Init
@@ -67,6 +67,15 @@ proc Sedit_Start { draft } {
 	set t [Widget_Text $f $sedit(height) -setgrid true -wrap char]
 
 	Drop_Attach $t SeditDragDrop
+
+	# PGP version-setting moved out from seditpgp code
+	if {$pgp(enabled)} {
+	    if {![info exists pgp($pgp(version,$id),myname,$id)]} {
+		set pgp($pgp(version,$id),myname,$id) $pgp($pgp(version,$id),myname)
+	    }
+	    EditAddPassPhrasePane $id .sedit$id
+	}
+
 	set sedit($t,status) [Widget_Entry .sedit${id} status {top fill} -relief raised]
 
 	# Nuke the Dismiss button because the Abort, Send, and Save&Quit
@@ -107,7 +116,7 @@ proc Sedit_Start { draft } {
 		$b.sign config -command $cmd
 	    }
 	} else {
-	    set menu [Widget_AddMenuBDef $b sign {right padx 1}]
+	    set menu [Widget_AddMenuBDef $b sign {right padx 1 filly}]
 	    set cmd [option get $b.sign command {}]
 	    set txt [option get $b.sign text {}]
 	    if ![string match *... $txt] {
@@ -138,9 +147,9 @@ proc Sedit_Start { draft } {
 	}
 	foreach but [Widget_GetButDef $b] {
 	    if {[regexp (abort|save) $but]} {
-		Widget_AddButDef $b $but {left padx 5}
+		Widget_AddButDef $b $but {left padx 5 filly}
 	    } else {
-		Widget_AddButDef $b $but {right padx 1}
+		Widget_AddButDef $b $but {right padx 1 filly}
 	    }
 	    Widget_ReEvalCmd $b.$but	;# expand variables now
 	}
@@ -148,7 +157,7 @@ proc Sedit_Start { draft } {
 	foreach M [Widget_GetMenuBDef .sedit$id.but] {
 	    global pgp
 	    if {$pgp(enabled) || ($M != "pgp")} {
-		set menu [Widget_AddMenuBDef .sedit$id.but $M {right padx 1}]
+		set menu [Widget_AddMenuBDef .sedit$id.but $M {right padx 1 filly}]
 		#
 		# Here is another way to set context for the menu commands.
 		# Jam the draft and text widget name into a global that
@@ -168,7 +177,7 @@ proc Sedit_Start { draft } {
 	set sedit(.sedit$id,draft) $draft
 	set sedit(.sedit$id,id) $id
     }
-    Exwin_ToplevelFocus .sedit$id $t
+    focus $t
 
     SeditTextBindings $draft $t		;# set up sendMsg binding
     if [file readable $quote(filename)] {
@@ -233,7 +242,7 @@ proc SeditSetContext { draft t } {
     Exmh_Status "Sedit $t [file tail $draft]"
 }
 proc SeditPositionCursor { t } {
-    global sedit tk_version
+    global sedit
     # Position cursor when the draft is first open.
     # Either on the first blank header line, or the first line of the message.
     # Body tag is assigned to the body and is used later when/if
@@ -250,9 +259,7 @@ proc SeditPositionCursor { t } {
 	    }
 	    if {! $header} {
 		$t mark set hlimit $l.end
-		if {$tk_version >= 4.0} {
-		    $t mark gravity hlimit left
-		}
+		$t mark gravity hlimit left
 		if {$l > 1} {incr l -1}
 		$t mark set header $l.end
 	    }
@@ -274,9 +281,7 @@ proc SeditPositionCursor { t } {
 	    # hlimit is used for <Tab> control
 	    # header is used to insert new header information
 	    $t mark set hlimit $l.end
-	    if {$tk_version >= 4.0} {
-		$t mark gravity hlimit left
-	    }
+	    $t mark gravity hlimit left
 	    if {$l > 1} {incr l -1}
 	    $t mark set header $l.end
 	    if {! $insert} {
@@ -304,7 +309,7 @@ or do nothing?"
 	Widget_Frame $f f Dialog
 	$f.f configure -bd 10
 	Widget_AddBut $f.f ok "Abort" [list SeditAbortDirect $draft $t]
-	Widget_AddBut $f.f send "Send\n(Ctrl-c)" [list SeditSend $draft $t]
+	Widget_AddBut $f.f send "Send\n(Ctrl-c)" [list SeditSend $draft $t 0]
 	Widget_AddBut $f.f save "Save\n(Ctrl-s)" \
 		[list SeditSave $draft $t SeditNuke]
 	Widget_AddBut $f.f no "Do nothing\n(Return)" [list destroy $f]
@@ -340,9 +345,9 @@ You might prefer Save&Quit."
     frame $f.but -bd 10 -relief flat
     pack $f.but -expand true -fill both
     set sedit($t,abort) nop
-    Widget_AddBut $f.but ok "Abort\n(Return)" [list SeditAbortConfirm $f $t abort] left
-    Widget_AddBut $f.but save "Save&Quit\n(Ctrl-s)" [list SeditAbortConfirm $f $t save] left
-    Widget_AddBut $f.but nop "Do Nothing\n(Ctrl-c)" [list SeditAbortConfirm $f $t nop] right
+    Widget_AddBut $f.but ok "Abort\n(Return)" [list SeditAbortConfirm $f $t abort] {left filly}
+    Widget_AddBut $f.but save "Save&Quit\n(Ctrl-s)" [list SeditAbortConfirm $f $t save] {left filly}
+    Widget_AddBut $f.but nop "Do Nothing\n(Ctrl-c)" [list SeditAbortConfirm $f $t nop] {right filly}
     bind $f.but <Return>    "$f.but.ok   flash ; $f.but.ok   invoke"
     bind $f.but <Control-c> "$f.but.nop  flash ; $f.but.nop  invoke"
     bind $f.but <Control-s> "$f.but.save flash ; $f.but.save invoke"
@@ -380,8 +385,9 @@ proc SeditMsg { t text } {
     update idletasks
 }
 
-proc SeditSend { draft t } {
+proc SeditSendCommon { draft t {post 0} } {
     global sedit exmh intelligentSign editor
+
     set id [SeditId $draft]
     SeditCheckForIsigHeaders $t
     Exmh_Debug SeditSend id=$id action=$exmh($id,action)
@@ -412,14 +418,22 @@ proc SeditSend { draft t } {
     }
     if [SeditSave $draft $t {} 0] {
 	global env sedit
-	$sedit($t,toplevel).but.send config -state disabled
+	if {$post==0} {
+	    $sedit($t,toplevel).but.send config -state disabled
+	} else {
+	    $sedit($t,toplevel).but.post config -state disabled
+	}
 	# Decide if this file needs to go through mhn
 	if {$sedit($t,mhn) && ![catch {exec grep -l ^# $draft}]} {
 	    set env(mhdraft) $draft
 	    SeditMsg $t "Running mhn..."
 	    if [catch {exec $editor(mhn) $draft} err] {
 		SeditMsg $t $err
-		$sedit($t,toplevel).but.send config -state normal
+		if {$post==0} {
+		    $sedit($t,toplevel).but.send config -state normal
+		} else {
+		    $sedit($t,toplevel).but.post config -state normal
+		}
 		return
 	    }
 	}
@@ -433,6 +447,16 @@ proc SeditSend { draft t } {
 	    # Insert content-transfer-encoding headers
 	    SeditFixupEncoding $draft $t [expr ($sedit($t,quote) > 0)]
 	}
+	return 1
+    } else {
+	return 0
+    }
+}
+
+proc SeditSendOnly { draft t } {
+    global sedit exmh
+
+	set id [SeditId $draft]
 	foreach cmd [info commands Hook_SeditSend*] {
 	    if [catch {$cmd $draft $t} err] {
 		SeditMsg $t "$cmd $err"
@@ -465,6 +489,21 @@ proc SeditSend { draft t } {
 	    }
 	    SeditMsg $t "Message saved and sent $time"
 	}
+}
+
+proc SeditSend { draft t {post 0} } {
+    global sedit exmh intelligentSign editor msg
+
+    set common [SeditSendCommon $draft $t $post]
+
+    if {$common==1} {
+	if {$post==0} {
+	    SeditSendOnly $draft $t
+	} else {
+	    set msg(path) $draft
+	    Post
+	    $sedit($t,toplevel).but.post config -state normal
+	}
     }
 }
 
@@ -560,8 +599,10 @@ proc Sedit_CheckPoint {} {
 }
 proc SeditPeriodicSave {} {
     global sedit
-    Sedit_CheckPoint
-    set sedit(checkpoint) [after [expr 1000 * 60] SeditPeriodicSave]
+    if { [info exists sedit(autosaveInterval)] && $sedit(autosaveInterval) > 0 } {
+        Sedit_CheckPoint
+        set sedit(checkpoint) [after [expr 1000 * $sedit(autosaveInterval) ] SeditPeriodicSave]
+    }
 }
 
 proc SeditFixupMhn { draft t } {
